/**
 * Copyright (c) 2016-2019 人人开源 All rights reserved.
 *
 * https://www.renren.io
 *
 * 版权所有，侵权必究！
 */

package com.common.utils;

import com.alibaba.fastjson.JSON;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * 返回数据
 *
 * @author Mark sunlightcs@gmail.com
 */
public class R<T> extends HashMap<String, Object> {
	private static final long serialVersionUID = 1L;
	/**
	 * 返回数据状态编码，为0表示成功
	 */
	private String errno;
	/**
	 * 返回数据说明
	 */
	private String errmsg;
	/**
	 * 返回数据
	 */
	private T result;
	public R() {
		put("errno","0");
		put("errmsg", "success");
	}

	public static R error() {
		return error("未知异常，请联系管理员");
	}

	public static R error(String msg) {
		return error("5001", msg);
	}

	public static R error(String code, String msg) {
		R r = new R();
		r.put("errno", code);
		r.put("errmsg", msg);
		return r;
	}

	public static R ok(String msg) {
		R r = new R();
		r.put("errmsg", msg);
		return r;
	}

	public static R ok(Map<String, Object> map) {
		R r = new R();
		r.putAll(map);
		return r;
	}

	public static R ok() {
		return new R();
	}

	@Override
	public R put(String key, Object value) {
		super.put(key, value);
		return this;
	}

	public R put(Object value) {
		put("result", JSON.toJSON(value));
		return this;
	}

	public R putEmptyList() {
		super.put("result", Collections.emptyList());
		return this;
	}
}
